// Versuch 1_2
// C. Roppel, HS Schmalkalden
// MATLAB- und Scilab-Übungen zum Buch "Grundlagen der Nachrichtentechnik", Hanser-Verlag, 2018
// Getestet mit Scilab 5.5.2
mode(0);
ieee(1);
clear
clc
xdel(winsid())

//**********
// Parameter
//**********
alpha = 0.5;  // Roll-off-Faktor
os = 8;       // Samples pro Symbol (Oversampling Ratio)
dly_rc = 3;   // Verzögerung des Wurzel-Kosinus-roll-off-Filters
nSym = 1000;  // Anzahl Symbole

//*******************************
// Wurzel-Kosinus-roll-off-Filter
//*******************************
t = -dly_rc*os:dly_rc*os;

psrc = (4*alpha*(t/os).*cos(%pi*(1 + alpha)*(t/os)) + sin(%pi*(1 - alpha)*(t/os)))./..
((1 - (4*alpha*(t/os)).^2)*%pi.*(t/os));
// Grenzwert für t/T_s = 0
psrc(dly_rc*os + 1) = (4*alpha + %pi*(1 - alpha))/%pi;
// Grenzwert für t/T_s = 1/(4 alpha))
if round(os/(4*alpha)) == os/(4*alpha) then  // Ist os/(4*alpha) ganzzahlig?
    psrc(dly_rc*os + 1 + os/(4*alpha)) = (4*alpha*cos(%pi*(1 + alpha)/(4*alpha)) - %pi*(1 + alpha)..
    *sin(%pi*(1 + alpha)/(4*alpha)) + %pi*(1 - alpha)*cos(%pi*(1 - alpha)/(4*alpha)))/(-2*%pi);
    psrc(dly_rc*os + 1 - os/(4*alpha)) = psrc(dly_rc*os + 1 + os/(4*alpha));
end

E = sum(psrc.^2);  // Energie
psrc = psrc/sqrt(E);  // Skalierung auf E = 1

//****************
// Basisbandsignal
//****************
grand("setsd", 1234);  // Initialisiere Zufallsgenerator
bits = grand(nSym, 1, "uin", 0, 1);  // Spaltenvektor der Länge nSym mit Zufallszahlen 0 oder 1
symbols = 2*bits - 1;  // Binäre bipolare Symbolfolge

symbols = upsample(symbols, os);  // Einfügen von os - 1 Nullen zwischen den Symbolen
x = conv(psrc, symbols);  // Faltung (convolution)
x = x(dly_rc*os + 1:$ - dly_rc*os);  // Entfernen der ersten und letzten Werte

figure;
plot(x(1:20*os));
xlabel("n");
ylabel("x(n)");

// Augendiagramm
figure;
eyemax = floor(length(x)/(2*os));
teye = ((-os:os)/os)';
for i = 1:eyemax - 1
    plot(teye, x(2*(i - 1)*os + 1:2*i*os + 1));
end
plot(teye(1:$-1), x(2*(eyemax - 1)*os + 1:2*eyemax*os));
xlabel("t/T_s");

// Leistungsdichtespektrum (Periodogramm)
len = length(x);
len = 2*floor(len/2);
x = x(1:len);
f = (os*(-len/2:len/2 - 1)/len)';
lds = 10*log10((1/(len*os))*fftshift(abs(fft(x)).^2));
figure;
plot(f, lds)
xlabel("f in Hz");
ylabel("LDS in dB");
